import { DetailQuiz } from "./DetailQuiz/detailQuiz";
import "./community.css";
import "../../index.css";
import { List } from "../../components/list/list";
import { useEffect, useState } from "react";
import { useQuizService } from "../../services/QuizService";
import HttpError from "../../services/HttpError";
import ListModel from "../../models/ListModel";
import { Quiz } from "../../models/Quiz.ts";
import { useSearchParams } from "react-router-dom";
import { Pagination } from "../../utils/pagination.tsx";

// page with the list component + the right side
export function Community() {
    const [search, setSearch] = useState<string>("");
    const [loading, setLoading] = useState<boolean>(false);
    const [quizList, setQuizList] = useState<Quiz[] | null>([]);
    const [selected, setSelected] = useState<string>("");
    const [liste, setListe] = useState<ListModel[]>([]);
    const [quiz, setQuiz] = useState<Quiz | null>();
    const [searchParams] = useSearchParams();
    const isMulti = searchParams.get("isMulti") === "true";
    const [page, setPage] = useState<number>(1);
    const [skip, setSkip] = useState<number>(0);
    const [total, setTotal] = useState<number>(0);
    const take = 10;
    const updateList = () => {
        useQuizService().getCommunityQuiz(skip, take, search).then((response: HttpError | { quizzes: Quiz[]; total: number; }) => {
            if (response instanceof HttpError) {
                console.log("Error while fetching quizzes:", response);
            } else {
                setTotal(response.total);
                setQuizList(response.quizzes);
                const tempList: ListModel[] = response.quizzes.map(quiz => ({
                    id: quiz.id,
                    title: quiz.name ?? "Unnamed Quiz",
                    content: `${quiz.questionCount} Questions | ${quiz.difficulty.name} | ${quiz.author.username ?? "Unnamed Author"}`,
                }));
                setListe(tempList);
            }
        });
    }

    useEffect(() => {
        setLoading(true);
        updateList();
        setLoading(false);
    }, []);

    useEffect(() => {
        setLoading(true);
        updateList();
        setLoading(false);
    }, [search, page]);

    useEffect(() => {
        if (selected && quizList?.length) {
            const selectedQuiz = quizList.find(quiz => quiz.id === selected.toString());
            setQuiz(selectedQuiz || null);
        }
    }, [selected]);

    const changePage = (changePage: number) => {
        setPage(page + changePage);
        setSkip(skip + take * changePage);
    }

    return !loading ? (
        <div className="container-right">
            <div className="left-side">
                <h1 className="h1">Community Quiz</h1>
                <input type="text" className="inputRecherche" placeholder="search a quiz..." onChange={(e) => { setSearch(e.currentTarget.value) }} />
                <div className="container-list">
                    <List list={liste} selected={selected} setSelected={setSelected} isLoading={loading} />
                </div>
                <Pagination page={page} pageSize={take} total={total} OnPageChange={(page: number) => { changePage(page) }} />
            </div>
            <div className="right-side">
                <DetailQuiz quiz={quiz} isMulty={isMulti} />
            </div>
        </div>
    ) : null;
}

interface QuizItem {
    id: string;
    title: string;
    content: string;
}
