// create quiz page
import { useEffect, useRef, useState } from "react";
import Swal from "sweetalert2";
import DeleteForeverOutlinedIcon from '@mui/icons-material/DeleteForeverOutlined';
import SaveOutlinedIcon from '@mui/icons-material/SaveOutlined';
import PublishOutlinedIcon from '@mui/icons-material/PublishOutlined';

import "./createQuiz.css";
import { useQuizService } from "../../services/QuizService";
import { Question, QuestionCreation } from "../../models/Question";
import { Category, Difficulty, Quiz, QuizCreation } from "../../models/Quiz";
import { AnswerCreation } from "../../models/Answer";
import { CreateAQuiz } from "../home/PlayAQuiz/createAQuiz";
import TextQuiz from "./textQuiz";
import ImageQuiz from "./imageQuiz";
import SoundQuiz from "./soundQuiz";
import HttpError from "../../services/HttpError";
import { useNavigate } from 'react-router-dom';


const EASY: Difficulty = { id: 1, name: "easy", }
const MEDIUM: Difficulty = { id: 2, name: "medium", }
const HARD: Difficulty = { id: 3, name: "hard", }
const QUESTIONS: QuestionCreation[] = [];

export function CreateQuiz() {
    const { createCommunityQuiz, getCategories, getQuestionsFromDatabase } = useQuizService();
    const [categories, setCategories] = useState<Category[]>([]);
    const [quiz, setQuiz] = useState<QuizCreation>(
        JSON.parse(localStorage.getItem("quiz")) ||
        {
            name: "",
            description: "Default description",
            difficulty: EASY.name,
            questions: QUESTIONS,
        });
    const [mode, setMode] = useState<string>("text");
    const navigate = useNavigate();

    useEffect(() => {
        localStorage.setItem("quiz", JSON.stringify(quiz));
        console.log("quiz saved in local storage", quiz);
    }, [quiz]);

    const fetchingCategories = async () => {
        const categories = await getCategories();
        let categoriesTemp: Category[] = [];
        categories.map((category) => {
            categoriesTemp.push({ name: category.name, id: category.id });
        });
        setCategories(categoriesTemp);
    };


    useEffect(() => {
        fetchingCategories();

        const lsQuiz = JSON.parse(localStorage.getItem("quiz") || null);
        if (lsQuiz) {
            setQuiz(lsQuiz);
        }
        if (quiz.questions.length === 0) {
            const initialQuestions = [createQuestion(quiz.questions.length)];
            setQuiz((prevQuiz) => ({
                ...prevQuiz,
                questions: initialQuestions,
            }));
        }
    }, []);

    // const updateQuiz = (e: any) => {
    //     e.preventDefault();
    //     setQuiz((prevQuiz) => ({
    //         ...prevQuiz,
    //         code: e.target.value,
    //     }));
    // }

    const createQuestion = (id: number): QuestionCreation => {
        const question: QuestionCreation = {
            text: "",
            answers: [],
            isMultipleChoice: true,
        };
        question.answers.push(createAnswer(true));
        question.answers.push(createAnswer());
        return question;
    };

    const addQuestion = () => {
        setQuiz((prevQuiz) => {
            const newQuestion = createQuestion(prevQuiz.questions.length);
            return {
                ...prevQuiz,
                questions: [...prevQuiz.questions, newQuestion],
            };
        });

    };

    const updateQuestion = (e: any, index: number) => {
        setQuiz((prevQuiz) => {
            const newQuestions = prevQuiz.questions.map((question, i: number) => {
                if (i === index) {
                    return {
                        ...question,
                        text: e.target.value, // Changed from 'question' to 'text'
                    };
                }
                return question;
            });
            return {
                ...prevQuiz,
                questions: newQuestions,
            };
        });

    };

    const createAnswer = (isCorrect?: boolean): AnswerCreation => {
        return {
            text: "",
            isCorrect: isCorrect || false,
        };
    };

    const addAnswer = (index: number) => {
        setQuiz((prevQuiz) => {
            // Clone questions array
            const newQuestions = prevQuiz.questions.map((question, i: number) => {
                // Find the matching question and clone it
                if (i === index) {
                    return {
                        ...question,
                        answers: [...question.answers, createAnswer()],
                    };
                }
                return question; // Keep other questions unchanged
            });

            // Return updated quiz state
            return {
                ...prevQuiz,
                questions: newQuestions,
            };
        });

    };

    const updateAnswer = (e: any, questionIndex: number, answerIndex: number) => {
        setQuiz((prevQuiz) => {
            const newQuestions = prevQuiz.questions.map((question, i: number) => {
                if (i === questionIndex) {
                    const newAnswers = question.answers.map((answer, j: number) => {
                        if (j === answerIndex) {
                            return {
                                ...answer,
                                text: e.target.value,
                            };
                        }
                        return answer;
                    });

                    return {
                        ...question,
                        answers: newAnswers,
                    };
                }
                return question;
            });

            return {
                ...prevQuiz,
                questions: newQuestions,
            };
        });

    };

    const handleDeleteAnswer = (e: any, questionIndex: number, answerIndex: number) => {
        e.preventDefault();
        setQuiz((prevQuiz) => {
            const newQuestions = prevQuiz.questions.map((question, i: number) => {
                if (i === questionIndex) {
                    const newAnswers = question.answers.filter((answer, j: number) => j !== answerIndex);
                    return {
                        ...question,
                        answers: newAnswers,
                    };
                }
                return question;
            });

            return {
                ...prevQuiz,
                questions: newQuestions,
            };
        });

    };

    const handleDeleteQuestion = (e: any, questionId: number) => {
        e.preventDefault();
        Swal.fire({
            title: "Are you sure?",
            text: "You won't be able to revert this!",
            icon: "warning",
            showCancelButton: true,
            confirmButtonColor: "#3085d6",
            cancelButtonColor: "#d33",
            confirmButtonText: "Yes, delete it!",
        }).then((result) => {
            if (result.isConfirmed) {
                deleteQuestion(questionId);
            }
        });
    };

    const deleteQuestion = (index: number) => {
        setQuiz((prevQuiz) => {
            const newQuestions = prevQuiz.questions.filter((question, i: number) => i !== index); // Supprime la question ciblée
            // .map((question, index) => ({
            //     ...question,
            //     id: index, // Réindexe les questions restantes
            // }));

            return {
                ...prevQuiz,
                questions: newQuestions,
            };
        });

    };

    const handlePopup = () => {
        const categoryOptions = categories
            .map((category) => `<option value="${category.id}">${category.name}</option>`)
            .join("");

        const popupForm: string = `
            <div class="container-caq">
                <form id="popupForm">
                    <div>
                        <label for="difficulty">Difficulty</label>
                        <select id="difficulty" class="form-control" name="difficulty">
                            <option value="easy">Easy</option>
                            <option value="medium">Medium</option>
                            <option value="hard">Hard</option>
                        </select>
                    </div>
                    <div>
                        <label for="category">Category</label>
                        <select id="category" class="form-control" name="category">
                            ${categoryOptions}
                        </select>
                    </div>
                    <div>
                        <label for="nbQuestions">Number of Questions</label>
                        <input
                            type="number"
                            id="nbQuestions"
                            class="form-control"
                            placeholder="Number of questions"
                            value="1"
                            name="nbQuestions"
                        />
                    </div>
                </form>
            </div>
        `;

        Swal.fire({
            title: "Add question from database",
            html: popupForm,
            showCloseButton: true,
            showCancelButton: true,
            confirmButtonText: "Submit",
            preConfirm: () => {
                const difficulty = (document.getElementById("difficulty") as HTMLSelectElement)?.value;
                const category = (document.getElementById("category") as HTMLSelectElement)?.value;
                const nbQuestions = (document.getElementById("nbQuestions") as HTMLInputElement)?.value;

                if (!difficulty || !category || !nbQuestions) {
                    Swal.showValidationMessage("Please fill out all fields");
                    return;
                }

                return {
                    difficulty: difficulty,
                    category: parseInt(category),
                    nbQuestions: parseInt(nbQuestions),
                };
            },
        }).then((result) => {
            if (result.isConfirmed) {
                const { difficulty, category, nbQuestions } = result.value!;
                addQuestionsFromDatabase(difficulty, category, nbQuestions);
            }
        });
    };

    const addQuestionsFromDatabase = (difficulty: string, category: number, nbQuestions: number) => {
        getQuestionsFromDatabase(category, difficulty, nbQuestions).then((result) => {
            const questions = result.questions;
            let newQuestions: any[] = [...quiz.questions];
            if (newQuestions[newQuestions.length - 1].text === "") {
                newQuestions = newQuestions.slice(0, newQuestions.length - 1);
            }

            if (Array.isArray(questions)) {
                newQuestions.push(
                    ...questions.map((question) => ({
                        text: question.text,
                        isMultipleChoice: question.type == "multiple",
                        answers: question.answers.map((answer) => ({
                            text: answer.text,
                            isCorrect: answer.isCorrect,
                        })),
                    }))
                );
                for (const question of newQuestions) {
                    const correctAnswerIndex = question.answers.findIndex((answer) => answer.isCorrect);
                    if (correctAnswerIndex > 0) {
                        const temp = question.answers[0];
                        question.answers[0] = question.answers[correctAnswerIndex];
                        question.answers[correctAnswerIndex] = temp;
                    }
                }
                setQuiz((prevQuiz) => ({
                    ...prevQuiz,
                    questions: newQuestions,
                }));

            } else {
                console.error("An error occurred while fetching questions from database :", result);
                Swal.fire("Error", "An error occurred while fetching questions from database", "error");
            }
        });
    };

    const saveQuiz = async () => {
        try {
            const difficultySelect = document.getElementById("difficulty") as HTMLSelectElement;
            switch (difficultySelect?.value) {
                case "easy":
                    quiz.difficulty = EASY.name;
                    break;
                case "medium":
                    quiz.difficulty = MEDIUM.name;
                    break;
                case "hard":
                    quiz.difficulty = HARD.name;
                    break;
                default:
                    quiz.difficulty = EASY.name;
                    break;
            }

            const titleInput = document.getElementById("title") as HTMLInputElement;
            if (titleInput) {
                quiz.name = titleInput.value;
            }

            Swal.fire({
                title: "publish the quiz ?",
                text: "You won't be able to edit it !",
                icon: "warning",
                showCancelButton: true,
                confirmButtonColor: "#3085d6",
                cancelButtonColor: "#d33",
                confirmButtonText: "Yes, publish it !",
            }).then((result) => {
                if (result.isConfirmed) {
                    const response = createCommunityQuiz(quiz);
                    if (response instanceof HttpError) {
                        Swal.fire("Error", response.message, "error");
                    } else {
                        Swal.fire("Success", "Quiz saved successfully!", "success");
                        localStorage.removeItem("quiz");
                        navigate("/profile");
                    }
                }
            });


        } catch (error) {
            Swal.fire("Error", error.message, "error");
        }
    };

    const HandleChangeMode = (e: any) => {
        setMode(e.target.value);
    }

    return (
        <>
            <div className="create-header">
                <h1>Create your own quiz</h1>
                <p className="blueIcon">
                    <PublishOutlinedIcon onClick={saveQuiz} />
                </p>
            </div>
            <div className="create-container">
                {/* <form> */}
                <div className="create-quiz-info">
                    <div className="create-quiz-info-wrapper">
                        <div className="create-label-wrapper">
                            <label>QUIZ TITLE </label>
                        </div>
                        <input
                            type="text"
                            id="title"
                            placeholder="Write the quiz title..." // onChange={(e) => updateQuiz}
                        />
                    </div>
                    <div className="create-quiz-info-wrapper">
                        <div className="create-label-wrapper">
                            <label>QUIZ DIFFICULTY </label>
                        </div>
                        <select id="difficulty">
                            <option value="easy">Easy</option>
                            <option value="medium">Medium</option>
                            <option value="hard">Hard</option>
                        </select>
                    </div>
                    <div className="create-quiz-info-wrapper">
                        <div className="create-label-wrapper">
                            <label>Quiz type</label>
                        </div>
                        <select id="quizType"
                            onChange={HandleChangeMode}>
                            <option value="text">Text</option>
                            <option value="image">Image - Coming soon</option>
                            <option value="sound">Sound - Coming soon</option>
                        </select>
                    </div>
                </div>
                <hr />
                <div className="create-question-info">
                    <div className="create-question-table">
                        {quiz.questions.map((question, indexQuestion) => (
                            <div className="create-question-row" key={indexQuestion}>
                                <div className="cell question-cell">
                                    <div className="create-label-wrapper">
                                        <label>QUESTION {indexQuestion + 1} </label>
                                    </div>
                                    <input
                                        type="text"
                                        id="question"
                                        placeholder="Write a question..."
                                        defaultValue={question.text}
                                        value={question.text}
                                        onChange={(e) => updateQuestion(e, indexQuestion)}
                                    />
                                    <p
                                        className="redIcon"
                                        onClick={(e) => {
                                            handleDeleteQuestion(e, indexQuestion);
                                        }}
                                    >
                                        <DeleteForeverOutlinedIcon />
                                    </p>
                                </div>
                                <div className="cell answers-cell" id="answers">
                                    <div className="create-label-wrapper">
                                        <label>ANSWERS </label>
                                    </div>
                                    <div className="answer-wrapper">
                                        <div className="answers-list">
                                            {mode === "text" && <TextQuiz answers={question.answers} indexQuestion={indexQuestion} updateAnswer={updateAnswer} handleDeleteAnswer={handleDeleteAnswer} />}
                                            {mode === "image" && <ImageQuiz answers={question.answers} indexQuestion={indexQuestion} updateAnswer={updateAnswer} handleDeleteAnswer={handleDeleteAnswer} />}
                                            {mode === "sound" && <SoundQuiz answers={question.answers} indexQuestion={indexQuestion} updateAnswer={updateAnswer} handleDeleteAnswer={handleDeleteAnswer} />}
                                        </div>
                                        <p className="error">All answers need to be unique.</p>
                                        {question.answers.length > 3 ? (
                                            <p>Maximum 4 answers.</p>
                                        ) : (
                                            <div className="add-cell">
                                                <button
                                                    onClick={(e) => {
                                                        e.preventDefault();
                                                        addAnswer(indexQuestion);
                                                    }}
                                                    className="questionsButton"
                                                >
                                                    + ADD AN ANSWER
                                                </button>
                                                <p>up to 4</p>
                                            </div>
                                        )}
                                    </div>
                                </div>
                                <hr />
                            </div>
                        ))}
                    </div>
                    <div className="btn-group">
                        <button
                            onClick={(e) => {
                                e.preventDefault();
                                addQuestion();
                            }}
                            className="questionsButton"
                        >
                            + ADD A QUESTION
                        </button>
                        <button
                            onClick={(e) => {
                                e.preventDefault();
                                handlePopup();
                            }}
                            className="questionsButton"
                        >
                            ADD FROM DATABASE
                        </button>
                    </div>
                </div>
                {/*</form>*/}
            </div>
        </>
    );
}
