// home page with title and 3 button, will use other components as right side
import { useState } from 'react';
import './home.css';
import { PlayAQuiz } from './PlayAQuiz/playAQuiz';
import { MyQuizzes } from './MyQuizzes/myQuizzes';
import { useQuizService } from '../../services/QuizService';
import { useNavigate } from 'react-router-dom';
import HttpError from "../../services/HttpError.ts";

enum HomeState {
    LOADING = 'LOADING',
    SHOWPAGE = 'SHOWPAGE',
}

export function Home() {
    const [paq, setPaq] = useState(false);  // Play A Quiz Selected
    const [mq, setMq] = useState(false);    // My Quizzes Selected
    const [homeState, setHomeState] = useState<HomeState>(HomeState.SHOWPAGE);
    const { getRandomQuiz } = useQuizService();
    const navigate = useNavigate();

    const showPaq = () => {
        setPaq(true);
        setMq(false);
    }

    const showMq = () => {
        setPaq(false);
        setMq(true);
    }

    const changeLoading = () => {
        homeState === HomeState.LOADING ? setHomeState(HomeState.SHOWPAGE) : setHomeState(HomeState.LOADING);
    }

    const quickGame = async () => {
        setPaq(false);
        setMq(false);

        setHomeState(HomeState.LOADING);
        const quizData = await getRandomQuiz()
        if (quizData instanceof HttpError) {
            console.error("Error while fetching random quiz:", quizData);
            return;
        }
        setHomeState(HomeState.SHOWPAGE);
        navigate('/quiz/' + quizData.id,  { state: { quizId: quizData.quizId } });
    }

    return (
        <>
            {homeState === HomeState.LOADING ? (

                <div className="center-loading-spinner-container">
                    <div className="center-loading-spinner"></div>
                </div>
            ) : (
                <div className="container-right">
                    <div className="left-side">
                        <img src="LogoApp.png" className="logo" />
                        {homeState === HomeState.SHOWPAGE && (
                            <div className="navigation-button">
                                <button onClick={quickGame}>Quick game</button>
                                <button onClick={showPaq}>Play a quiz</button>
                                <button onClick={showMq}>My quizzes</button>
                            </div >
                        )}

                    </div >
                    <div className="right-side">
                        {paq && <PlayAQuiz loading={changeLoading} />}
                        {mq && <MyQuizzes loading={changeLoading} />}
                    </div >
                </div>
            )}
        </>
    )
}
