import { useState } from "react";
import Swal from "sweetalert2";
import "./lobby.css";
import { useNavigate, useParams } from "react-router-dom";
import { Quiz } from "../../../models/Quiz";
import { useMultiplayerService } from "../../../services/MultiplayerService";
import QRCode from "qrcode";
import { User } from "../../../models/User";

export function Lobby() {
    const { id } = useParams<{ id: string }>();
    const [quiz, setQuiz] = useState<Quiz>();
    const [isHost, setIsHost] = useState<boolean>(false);
    const [players, setPlayers] = useState<User[]>([
        { username: "Alice", email: "test@test.com", id: 12, stats: null },
        { username: "Bob", email: "test@test.com", id: 13, stats: null },
        { username: "Paul", email: "test@test.com", id: 14, stats: null },
        { username: "Eduard", email: "test@test.com", id: 15, stats: null },
    ]);
    const { startParty, recivedPlayers, joinLobby } = useMultiplayerService();
    const navigate = useNavigate();

    const onInit = async () => {
        const res = await joinLobby(id);
        setIsHost(res.isHost);
    };

    const onChangePlayer = async () => {
        const players = await recivedPlayers();
        setPlayers(players);
    };

    const leave = async () => {
        navigate("/multiplayer");
    };

    const showInvitation = async () => {
        const currentUrl = window.location.href;
        const qrCodeUrl = await QRCode.toDataURL(currentUrl);

        Swal.fire({
            title: "Invitation",
            html: `
                <p>Invite tes amis à jouer avec toi dans cette partie !</p>
                <img src="${qrCodeUrl}" alt="QR Code" style="width: 200px; height: 200px; border-radius: 0%;" />
                <p>${currentUrl}</p>
            `,
            showCancelButton: true,
            confirmButtonText: "COPY LINK",
            cancelButtonText: "CLOSE",
        }).then((result) => {
            if (result.isConfirmed) {
                navigator.clipboard.writeText(currentUrl);
                Swal.fire("Copied!", "", "success");
            }
        });
    };

    const cancel = async () => {
        navigate("/multiplayer");
    };

    const play = async () => {
        startParty(id);
    };

    return (
        <div className="container-right">
            <div className="left-side">
                <div className="player-list">
                    {players.map((player, index) => (
                        <div className="player" key={index}>
                            <img
                                src="https://cdn.pixabay.com/photo/2015/10/05/22/37/blank-profile-picture-973460_1280.png"
                                alt="profile picture"
                                className="player-image"
                            />
                            <p className="player-name">{player.username}</p>
                        </div>
                    ))}
                </div>
                <div className="new-quiz">
                    <button className="blueButton" onClick={showInvitation}>
                        INVITATION
                    </button>
                </div>
            </div>
            <div className="right-side">
                <div className="quiz-info">
                    <div className="quiz-title">
                        <p className="quiz-info-title">Quiz information</p>
                        <hr />
                    </div>

                    <div className="quiz-info-content">
                        <p className="quiz-info-content-value">{quiz?.category?.name}</p>
                    </div>
                    <div className="quiz-info-content">
                        <p className="quiz-info-content-title">Nb. questions</p>
                        <p className="quiz-info-content-value">{quiz?.questionCount}</p>
                    </div>
                </div>

                <div className="new-quiz">
                    {!isHost ? (
                        <button className="blueButton" onClick={leave}>
                            LEAVE
                        </button>
                    ) : (
                        <div className="host-buttons">
                            <button className="greenButton" onClick={play}>
                                PLAY
                            </button>
                            <button className="redButton" onClick={cancel}>
                                CANCEL
                            </button>
                        </div>
                    )}
                </div>
            </div>
        </div>
    );
}
