// To join a quiz by id
import { useState } from "react";
import "./joinAQuiz.css";
import { useQuizService } from "../../../services/QuizService.ts";
import { useNavigate } from "react-router-dom";
import { Quiz } from "../../../models/Quiz.ts";
import { useMultiplayerService } from "../../../services/MultiplayerService.ts";

interface paqProps {
    loading: () => void;
}

export function JoinAQuiz({ loading }: paqProps) {
    const [lobbyFound, setLobbyFound] = useState(false);
    const [lobby, setLobby] = useState<any>();
    const [errMassage, setErrMassage] = useState<string>("No quiz found");
    const { lobbyInfos } = useMultiplayerService();
    const navigate = useNavigate();

    const changeIDQuiz = async (event: any) => {
        const idLobby = event.target.value;
        if (idLobby.length === 6) {
            const lobby = await lobbyInfos(idLobby);
            if (lobby instanceof Error) {
                console.error("Error while fetching lobby:", lobby);
                setErrMassage(lobby.message);
                return;
            }
            setLobby(lobby);
            setLobbyFound(true);
        } else {
            setLobbyFound(false);
            setErrMassage("No quiz found");
        }
    };

    const handleGo = () => {
        if (!lobbyFound) {
            return;
        } else {
            navigate("/lobby/" + lobby.id);
        }
    };

    return (
        <div className="container-jaq">
            <p className="title">Join a quiz</p>
            <form>
                <div className="form-group">
                    <input type="text" className="inputRecherche" placeholder="code..." onChange={changeIDQuiz} />
                </div>
            </form>
            <div className="quiz-info">
                <div className="quiz-title">
                    <p className="quiz-info-title">Quiz information</p>
                    <hr />
                </div>
                {lobbyFound ? (
                    <>
                        <div className="quiz-info-content">
                            <p className="quiz-info-content-value">{lobby.category.name}</p>
                        </div>
                        <div className="quiz-info-content">
                            <p className="quiz-info-content-title">Nb. questions</p>
                            <p className="quiz-info-content-value">{lobby.questionCount}</p>
                        </div>
                    </>
                ) : (
                    <div className="quiz-info-content">
                        <p className="quiz-info-content-value">{errMassage}</p>
                    </div>
                )}
            </div>

            <div className="new-quiz">
                <button className="blueButton" onClick={handleGo}>
                    Go !
                </button>
            </div>
        </div>
    );
}
