import "./endQuizModal.css";
import { ModalButton } from "../../../components/quiz/modalButton/modalButton.tsx";
import { useNavigate } from "react-router-dom";
import { MultiplayerQuizInformations } from "../../../models/QuizInformations.ts";

interface Props {
    isOpen: boolean;
    onClose: () => void;
    quizInformations: MultiplayerQuizInformations;
    score: number;
}

export const EndQuizModal = ({ isOpen, onClose, quizInformations, score }: Props) => {
    if (!isOpen) return null;
    const navigate = useNavigate();

    const handleBackToMenu = () => {
        navigate("/multiplayer");
    };
    return (
        <div className="end-quiz-modal-overlay">
            <div className="end-quiz-modal-content" onClick={(e) => e.stopPropagation()}>
                <div>
                    <p className="end-quiz-modal-button-title">CONGRATS !</p>
                </div>
                <div className="end-quiz-modal-button-text-container">
                    <p className="end-quiz-modal-button-text">YOU FINISH THE TEST WITH A</p>
                    <p className="end-quiz-modal-button-text">
                        SCORE OF {score}/{quizInformations.questionCount} !
                    </p>
                </div>
                <div>
                    <div className="end-quiz-modal-scores">
                        <img src="stars.png" alt="stars picture" className="stars-image" />
                        {quizInformations.scores.slice(0, 3).map((score, index) => (
                            <div key={index} className="score-item">
                                <div className="score-avatar">
                                    <img
                                        src="https://cdn.pixabay.com/photo/2015/10/05/22/37/blank-profile-picture-973460_1280.png"
                                        alt="profile picture"
                                        className="player-image"
                                    />
                                </div>
                                <p>{quizInformations.names[index]}</p>
                                <p>
                                    {score}/{quizInformations.questionCount}
                                </p>
                            </div>
                        ))}
                        <img src="stars.png" alt="stars picture" className="stars-image-reverse" />
                    </div>
                    <div className="end-quiz-modal-scores">
                        {quizInformations.scores.slice(3, 8).map((score, index) => (
                            <div key={index} className="score-item">
                                <p>{quizInformations.names[index + 3]}</p>
                                <p>{score}/20</p>
                            </div>
                        ))}
                    </div>
                </div>

                <div className="end-quiz-modal-button-container">
                    {/*<ModalButton text="RESTART" onClick={onClose} isLeftButton={true} isDisabled={false}/>*/}
                    <ModalButton
                        text="BACK TO MENU"
                        onClick={handleBackToMenu}
                        isLeftButton={false}
                        isDisabled={false}
                    />
                </div>
            </div>
        </div>
    );
};
