import "./quiz.css";
import { useEffect, useState } from "react";
import { useMultiplayerService } from "../../../services/MultiplayerService.ts";
import HttpError from "../../../services/HttpError.ts";
import { QuizTop } from "./quizTop/quizTop.tsx";
import { QuizBot } from "./quizBot/quizBot.tsx";
import { EndQuizModal } from "../endQuizModal/endQuizModal.tsx";
import { MultiplayerQuizInformations } from "../../../models/QuizInformations.ts";

enum QuizState {
    LOADING = "Loading",
    ERROR = "Error",
    OK = "Ok",
}
export function PlayingQuiz() {
    const [question, setQuestion] = useState<any>();
    const { recivedQuestion, recivedResults, recivedEnd, sendAnswer } = useMultiplayerService();
    const [quizState, setQuizState] = useState(QuizState.LOADING);
    const [errorMessage, setErrorMessage] = useState("");
    const [endQuizModalIsOpen, setEndQuizModalIsOpen] = useState(false);
    const [quizInformations, setQuizInformations] = useState<MultiplayerQuizInformations>();
    const [score, setScore] = useState(0);

    const fetchQuiz = async () => {
        const question = await recivedQuestion();
        if (question instanceof HttpError) {
            setQuizState(QuizState.ERROR);
            setErrorMessage(question.message);
            return;
        }
        if (question.questionIndex >= question.questionCount) {
            setQuizState(QuizState.ERROR);
            setErrorMessage("You already finished the quiz");
            return;
        }
        setQuizState(QuizState.OK);
        setQuestion(question);
    };

    const recivedEndQuiz = async () => {
        const end = await recivedEnd();
        if (end instanceof HttpError) {
            setQuizState(QuizState.ERROR);
            setErrorMessage(end.message);
            return;
        }
        setQuizInformations(end.informations);
        setScore(end.score);
        setEndQuizModalIsOpen(true);
    };

    useEffect(() => {
        fetchQuiz();
    }, []);

    return (
        <div className="quiz-container">
            {quizState === QuizState.LOADING && (
                <div className="center-loading-spinner-container">
                    <div className="center-loading-spinner"></div>
                </div>
            )}
            {quizState === QuizState.ERROR && <h2>Error: {errorMessage}</h2>}
            {quizState === QuizState.OK && question && (
                <>
                    <QuizTop question={question} />
                    <QuizBot question={question} />
                    <EndQuizModal
                        isOpen={endQuizModalIsOpen}
                        onClose={null}
                        quizInformations={quizInformations}
                        score={score}
                    />
                </>
            )}
        </div>
    );
}
