import './quiz.css';
import {useLocation, useParams} from "react-router-dom";
import {useEffect, useState} from "react";
import {useQuizService} from "../../services/QuizService.ts";
import HttpError from "../../services/HttpError.ts";
import { QuizTop } from "./quizTop/quizTop.tsx";
import { QuizBot } from "./quizBot/quizBot.tsx";
import { EndQuizModal } from "../../components/quiz/endQuizModal/endQuizModal.tsx";
import { StopQuizModal } from "../../components/quiz/stopQuizModal/stopQuizModal.tsx";
import {Quiz} from "../../models/Quiz.ts";
import {Question} from "../../models/Question.ts";
import {QuizInformations} from "../../models/QuizInformations.ts";
import {shuffleAnswers} from "../../utils/QuizHelper.ts";

type QuizParams = {
    id: string;
};

enum QuizState {
    LOADING = "Loading",
    ERROR = "Error",
    OK = "Ok",
}
export function PlayingQuiz() {
    const [endQuizModalIsOpen, setEndQuizModalIsOpen] = useState(false);
    const [stopQuizModalIsOpen, setStopQuizModalIsOpen] = useState(false);
    const { id } = useParams<QuizParams>();
    const [quiz, setQuiz] = useState<Quiz>();
    const { getQuizInformations, getActualQuestionOfAQuiz } = useQuizService();
    const [quizState, setQuizState] = useState(QuizState.LOADING);
    const [errorMessage, setErrorMessage] = useState("");
    const [quizInformations, setQuizInformations] = useState<QuizInformations | undefined>(undefined);
    const [actualQuestion, setActualQuestion] = useState<Question | undefined>(undefined);
    const [score, setScore] = useState(0);

    const location = useLocation();
    const quizId = location.state?.quizId;

    const fetchQuizInformations = async () => {
        const quizInformationsFetched = await getQuizInformations(quizId);
        if (HttpError.isHttpError(quizInformationsFetched)) {
            return;
        }
        setQuizInformations(quizInformationsFetched);
    }

    const fetchActualQuestion = async () => {
        const actualQuestionFetched = await getActualQuestionOfAQuiz(id);
        if (HttpError.isHttpError(actualQuestionFetched)) {
            return;
        }
        const actualQuestionFetchedShuffled = shuffleAnswers(actualQuestionFetched);
        setActualQuestion(actualQuestionFetchedShuffled);
    }

    useEffect(() => {
        fetchQuizInformations();
    }, []);

    useEffect(() => {
        fetchActualQuestion();
        setQuizState(QuizState.OK);
    }, [quizId]);


    return (
        <div className="quiz-container">
            {quizState === QuizState.LOADING && (
                <div className="center-loading-spinner-container">
                    <div className="center-loading-spinner"></div>
                </div>
            )}
            {quizState === QuizState.ERROR && (
                <h2>Error: {errorMessage}</h2>
            )}
            {quizState === QuizState.OK && quizInformations && actualQuestion && (
                <>
                    <QuizTop quizInformations={quizInformations} actualQuestion={actualQuestion} score={score} setStopQuizModalIsOpen={setStopQuizModalIsOpen} />
                    <QuizBot quizInformations={quizInformations} runId={id} actualQuestion={actualQuestion} fetchActualQuestion={fetchActualQuestion} setScore={setScore} OpenModalEndQuiz={setEndQuizModalIsOpen} />
                    <StopQuizModal isOpen={stopQuizModalIsOpen} onClose={() => setStopQuizModalIsOpen(false)} />
                    <EndQuizModal isOpen={endQuizModalIsOpen} setIsOpen={setEndQuizModalIsOpen} quizInformations={quizInformations} score={score} />
                </>
            )}
        </div>
    );
}
