import HttpError from "./HttpError";
import axios from "axios";
import {User} from '../models/User.ts';

export const useUserService =  () => {
    const url = "https://klebert-host.com:33037";


    const loginUser = async (login: string, password: string): Promise<boolean | HttpError> => {
        try {
            const requestBody = {
                login: login,
                password: password,
            };

            await axios.post(`${url}/auth/login`, requestBody, {
                withCredentials: true, // Autorise Axios à inclure les cookies
            });
            return true;
        } catch (error: any) {
            if (error.response) {
                // Si le mot de passe ou le connect est incorrect
                if (error.response.status === 401) {
                    return new HttpError(401, "Invalid email or password");
                }

                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                // Erreur réseau ou autre
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    };

    const getInformationsUser = async (): Promise<User | HttpError> => {
        try {
            const response = await axios.get<User>(`${url}/user/me`, {
                withCredentials: true, // Inclut les cookies dans la requête
            });


            return response.data;
        } catch (error: any) {
            if (error.response.status === 401) {
                return new HttpError(401, "No user logged in");
            }

            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    };

    const logoutUser = async (): Promise<boolean | HttpError> => {
        try {
            await axios.post(`${url}/auth/logout`, null, {
                withCredentials: true, // Inclut les cookies dans la requête
            });
            return true;
        } catch (error: any) {
            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }

    const registerUser = async (
        username: string,
        password: string,
        email: string
    ): Promise<boolean | HttpError> => {
        try {
            const requestBody = {
                username: username,
                email: email,
                password: password,
            };

            // Enregistrement de l'utilisateur
            await axios.post(`${url}/user`, requestBody, {
                withCredentials: true,
            });

            // Connexion après enregistrement
            // const loginResult = await loginUser(username, password);
            //
            // if (loginResult instanceof HttpError) {
            //     return loginResult; // Retourne l'erreur de connexion si échec
            // }

            return true; // Succès
        } catch (error: any) {
            if(error.response.status === 400) {
                return new HttpError(400, "User already exists");
            }
            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message);
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    };




    return {
        loginUser: loginUser,
        getInformationsUser: getInformationsUser,
        logoutUser: logoutUser,
        registerUser: registerUser
    };
}
